"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RabbitMQBundle = void 0;
const core_1 = require("@bluelibs/core");
const ejson_1 = require("@bluelibs/ejson");
const amqplib_1 = require("amqplib");
class RabbitMQBundle extends core_1.Bundle {
    constructor() {
        super(...arguments);
        this.defaultConfig = {
            url: "amqp://localhost:5672/",
            consume: true,
        };
        this.assertedQueues = [];
    }
    async init() {
        this.connection = await amqplib_1.connect(this.config.url);
        this.channel = await this.connection.createChannel();
    }
    async assertQueue(queue, options) {
        return this.channel.assertQueue(queue, options);
    }
    /**
     * Publish the message to the queue
     *
     * @param queue
     * @param message
     * @param options
     */
    publish(queue, message, options) {
        return this.channel.sendToQueue(queue, Buffer.from(ejson_1.EJSON.stringify(message)), options);
    }
    /**
     * Consume a message
     * @param queue
     * @param handler
     * @param options
     */
    consume(queue, handler, options) {
        if (!this.config.consume) {
            return;
        }
        this.channel.consume(queue, async (msg) => {
            await handler(ejson_1.EJSON.parse(msg.content.toString()));
            if (!options.noAck) {
                this.channel.ack(msg);
            }
        }, options);
    }
    async shutdown() {
        await this.channel.close();
        await this.connection.close();
    }
}
exports.RabbitMQBundle = RabbitMQBundle;
//# sourceMappingURL=RabbitMQBundle.js.map